from threading import local


class NavigationRootInfo(local):
    root = None

_current_root = NavigationRootInfo()


def getNavigationRoot():
    """Get the current navigation root
    """
    return _current_root.root


def setNavigationRoot(root):
    """Set the current navigation root. This is normally done by an event
    subscriber during traversal.
    """
    _current_root.root = root



from collective.rooter.navroot import setNavigationRoot
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.component import adapter
from zope.publisher.interfaces import IEndRequestEvent
from zope.traversing.interfaces import IBeforeTraverseEvent


@adapter(INavigationRoot, IBeforeTraverseEvent)
def record_navigation_root(obj, event):
    """When traversing over a site manager that is a navigation root,
    record the navigation root in a thread-local.
    """
    setNavigationRoot(obj)


@adapter(IEndRequestEvent)
def clean_navigation_root(event):
    """When traversal is over, clear the navigation root thread-local
    """
    setNavigationRoot(None)



from Products.CMFPlone.CatalogTool import CatalogTool
from collective.rooter.navroot import getNavigationRoot


# Make portal_catalog's search function use a navigation root if no path
# parameter is given
CatalogTool._oldSearchResults = CatalogTool.searchResults


def searchResults(self, REQUEST=None, **kw):
    if 'UID' not in kw\
            and 'path' not in kw\
            and (REQUEST is None or 'path' not in REQUEST):
        # Root catalog searches to the navigation root, except if path or UID
        # are explicitly given.
        root = getNavigationRoot()
        if root is not None:
            kw = kw.copy()
            kw['path'] = '/'.join(root.getPhysicalPath())
    return CatalogTool._oldSearchResults(self, REQUEST, **kw)



from Products.CMFCore.utils import getToolByName
from Products.Five import fiveconfigure
from Products.Five import zcml
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup
from collective.rooter import getNavigationRoot
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.uuid.utils import uuidToObject
from plone.uuid.interfaces import IUUID
from zope.app.publication.interfaces import BeforeTraverseEvent
from zope.event import notify
from zope.interface import alsoProvides
import unittest


@onsetup
def setup_package():
    fiveconfigure.debug_mode = True
    import collective.rooter
    zcml.load_config('configure.zcml', collective.rooter)
    fiveconfigure.debug_mode = False

setup_package()
ptc.setupPloneSite()


class IntegrationTests(ptc.PloneTestCase):

    def test_search_root(self):
        self.setRoles(['Manager'])
        self.portal.invokeFactory('Folder', 'site1')
        alsoProvides(self.portal.site1, INavigationRoot)

        self.portal.invokeFactory('Document', 'd1')
        self.portal.site1.invokeFactory('Document', 'd2')

        catalog = getToolByName(self.portal, 'portal_catalog')

        lazy = catalog(portal_type='Document')
        results = [x.getId for x in lazy]
        self.failUnless('d1' in results)
        self.failUnless('d2' in results)

        # Simulate traversal
        notify(BeforeTraverseEvent(self.portal.site1, self.portal.REQUEST))

        root = getNavigationRoot()
        self.assertEquals(
            root.absolute_url(),
            self.portal.site1.absolute_url())

        lazy = catalog(portal_type='Document')
        results = [x.getId for x in lazy]
        self.failIf('d1' in results)
        self.failUnless('d2' in results)

    def test_search_root_with_explicit_path(self):
        self.setRoles(['Manager'])
        self.portal.invokeFactory('Folder', 'site1')
        alsoProvides(self.portal.site1, INavigationRoot)

        self.portal.invokeFactory('Document', 'd1')
        self.portal.site1.invokeFactory('Document', 'd2')

        catalog = getToolByName(self.portal, 'portal_catalog')

        lazy = catalog(portal_type='Document',
                       path='/'.join(self.portal.getPhysicalPath()))
        results = [x.getId for x in lazy]
        self.failUnless('d1' in results)
        self.failUnless('d2' in results)

        # Simulate traversal
        notify(BeforeTraverseEvent(self.portal.site1, self.portal.REQUEST))

        lazy = catalog(portal_type='Document',
                       path='/'.join(self.portal.getPhysicalPath()))
        results = [x.getId for x in lazy]
        self.failUnless('d1' in results)
        self.failUnless('d2' in results)

    def test_get_obj_by_uuid(self):
        """uuidToObject should also return objects from other subsites, if the
        UUID is known and permissions allow to.
        """
        self.setRoles(['Manager'])
        self.portal.invokeFactory('Folder', 'site1')
        alsoProvides(self.portal.site1, INavigationRoot)

        self.portal.invokeFactory('Document', 'd1')
        d1 = self.portal['d1']

        # Simulate traversal
        notify(BeforeTraverseEvent(self.portal.site1, self.portal.REQUEST))

        root = getNavigationRoot()
        self.assertEquals(
            root.absolute_url(),
            self.portal.site1.absolute_url())

        self.assertEqual(uuidToObject(IUUID(d1)), d1)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(IntegrationTests))
    return suite



from setuptools import find_packages
from setuptools import setup

version = '1.0'

setup(
    name='collective.rooter',
    version=version,
    description=u"Tools to force catalog queries to obey the current "
                u"navigation root",
    long_description="%s\n%s" % (
        open("README.rst").read(),
        open("CHANGES.rst").read()
    ),
    classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='plone navigation root',
    author='Martin Aspeli',
    author_email='optilude@gmail.com',
    url='http://pypi.python.org/pypi/collective.rooter',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'Products.CMFPlone',
        'collective.monkeypatcher',
        'plone.app.layout',
        'zope.component',
        'zope.publisher',
        'zope.traversing',
    ],
    extras_require=dict(
        test=[
            'Products.PloneTestCase',
            'plone.app.uuid',
            'plone.uuid',
        ],
    ),
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
